package w2phtml.rdf;

import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.opencsv.CSVReaderHeaderAware;
import com.opencsv.exceptions.CsvValidationException;

public class MetadataContainer {
	private static final String SUBTITLE = "subtitle";
	private static final String FILENAME = "Filename";
	private static final String SECTION = "Section";
	private static final Logger logger  = LoggerFactory.getLogger(MetadataContainer.class);
	
	private HashMap<String, ArrayList<Map<String, String>>> sectionsMetadata;

	public MetadataContainer() {
		sectionsMetadata = new HashMap<String, ArrayList<Map<String, String>>>();
		 
	}

	public void read(String filePath) {
		if (filePath == null) {
			return;
		}
		File file = new File(filePath);
		if (!file.exists() || !file.canRead()) {
			return;
		}
	  readCSVFile(filePath);
	}

	private void readCSVFile(String filePath) {
		Map<String, String> lineMap;
		CSVReaderHeaderAware csvReader = null;
  	int i = 0;
		try {		
	  	FileReader reader = new FileReader(filePath);
			csvReader = new CSVReaderHeaderAware(reader);
			while (i >= 0) {
	  		lineMap = csvReader.readMap();
	  		if (lineMap != null) {
	  			processCSVLine(lineMap);
	  		} else {
	  			break;
	  		}
	  		i++;
	  	} 
			
		} catch (CsvValidationException e) {
			logger.error("CSV Validation failed. File " + filePath);
			logger.error(e.toString());
			StringWriter sw = new StringWriter();
			e.printStackTrace( new PrintWriter(sw));
			logger.error(sw.toString());
		} catch (IOException e) {
			logger.error("Bad CSV file " + filePath);
			logger.error(e.toString());
		} finally {
			try {
				csvReader.close();
			} catch (IOException e) {
				logger.error("On closing reader I/O Exception happened.");
			}
		}
	}

	private void processCSVLine(Map<String, String> lineMap) {
		Set<String> keys = lineMap.keySet();
		if (!keys.contains(SECTION)) {
			processDocumentMetadata(lineMap);
		} else {
			String section = lineMap.get(SECTION);
			processSectionMetadata(lineMap,section);
		}
	}

	private void processDocumentMetadata(Map<String, String> lineMap) {
		processSectionMetadata(lineMap, "");
	}

	private void processSectionMetadata(Map<String, String> lineMap, String sectionNum) {
		ArrayList<Map<String, String>> sectionMetadata = getSection(sectionNum);
		Iterator<Entry<String, String>> iterator = lineMap.entrySet().iterator();
		for (;iterator.hasNext();) {
			Entry<String, String> entry = iterator.next();
			String key = entry.getKey();
			String value = entry.getValue();
			if (key == null || value == null || key.trim().isEmpty() || value.trim().isEmpty()) {
				iterator.remove();
			}
		}
		sectionMetadata.add(lineMap);
	
	}

	public ArrayList<Map<String, String>> getSection(String sectionNum) {
		if (sectionNum.equals("0")) {
			sectionNum = "";
		}
		if (!sectionsMetadata.containsKey(sectionNum)) {
			ArrayList<Map<String, String>> sectionMetadata = new ArrayList<Map<String, String>>();
			sectionsMetadata.putIfAbsent(sectionNum, sectionMetadata);
		}
		return sectionsMetadata.get(sectionNum);
	}

	public String getFirstValue(String section, String metadataName) {
		if (section.equals("0")) {
			section = "";
		}
		if (sectionsMetadata.containsKey(section)) {
			ArrayList<Map<String, String>> sectionMetadata = sectionsMetadata.get(section);
			for (Map<String, String> lineMap : sectionMetadata) {
				String value = lineMap.get(metadataName);
				if (value != null && !value.trim().isEmpty()) {
					return value;
				}
			}
		}
		return "";
	}
	public void addMetadata(String section, String metaName, String metaValue) {
		if (section.equals("0")) {
			section = "";
		}
		ArrayList<Map<String, String>> sectionContainer = getSection(section);
		HashMap<String, String> map = new HashMap<String,String>();
		map.put(metaName, metaValue);
		sectionContainer.add(map);
	}

}
